@@ -3,12 +3,12 @@
 
 from __future__ import annotations
 
-from typing import Callable, Hashable, Sequence
+from typing import Callable, Hashable, Sequence, Any
 
-from rapidfuzz._common_py import conv_sequences
-from rapidfuzz._utils import is_none
+from rapidfuzz._common_py import common_affix, conv_sequences
+from rapidfuzz._utils import is_none, setupPandas
 from rapidfuzz.distance import Indel_py as Indel
-from rapidfuzz.distance._initialize_py import Editops, Opcodes
+from rapidfuzz.distance._initialize_py import Editop, Editops, Opcodes
 
 
 def _levenshtein_maximum(s1: Sequence[Hashable], s2: Sequence[Hashable], weights: tuple[int, int, int]) -> int:
@@ -100,7 +100,7 @@ def distance(
         First string to compare.
     s2 : Sequence[Hashable]
         Second string to compare.
-    weights : Tuple[int, int, int] or None, optional
+    weights : tuple[int, int, int] or None, optional
         The weights for the three operations in the form
         (insertion, deletion, substitution). Default is (1, 1, 1),
         which gives all three operations a weight of 1.
@@ -184,7 +184,7 @@ def similarity(
         First string to compare.
     s2 : Sequence[Hashable]
         Second string to compare.
-    weights : Tuple[int, int, int] or None, optional
+    weights : tuple[int, int, int] or None, optional
         The weights for the three operations in the form
         (insertion, deletion, substitution). Default is (1, 1, 1),
         which gives all three operations a weight of 1.
@@ -245,7 +245,7 @@ def normalized_distance(
         First string to compare.
     s2 : Sequence[Hashable]
         Second string to compare.
-    weights : Tuple[int, int, int] or None, optional
+    weights : tuple[int, int, int] or None, optional
         The weights for the three operations in the form
         (insertion, deletion, substitution). Default is (1, 1, 1),
         which gives all three operations a weight of 1.
@@ -271,6 +271,7 @@ def normalized_distance(
         If unsupported weights are provided a ValueError is thrown
     """
     _ = score_hint
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 1.0
 
@@ -307,7 +308,7 @@ def normalized_similarity(
         First string to compare.
     s2 : Sequence[Hashable]
         Second string to compare.
-    weights : Tuple[int, int, int] or None, optional
+    weights : tuple[int, int, int] or None, optional
         The weights for the three operations in the form
         (insertion, deletion, substitution). Default is (1, 1, 1),
         which gives all three operations a weight of 1.
@@ -358,6 +359,7 @@ def normalized_similarity(
     0.81818181818181
     """
     _ = score_hint
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0.0
 
@@ -372,6 +374,47 @@ def normalized_similarity(
     return norm_sim if (score_cutoff is None or norm_sim >= score_cutoff) else 0
 
 
+def _matrix(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> tuple[int, list[int], list[int]]:
+    if not s1:
+        return (len(s2), [], [])
+
+    VP = (1 << len(s1)) - 1
+    VN = 0
+    currDist = len(s1)
+    mask = 1 << (len(s1) - 1)
+
+    block: dict[Hashable, int] = {}
+    block_get = block.get
+    x = 1
+    for ch1 in s1:
+        block[ch1] = block_get(ch1, 0) | x
+        x <<= 1
+
+    matrix_VP = []
+    matrix_VN = []
+    for ch2 in s2:
+        # Step 1: Computing D0
+        PM_j = block_get(ch2, 0)
+        X = PM_j
+        D0 = (((X & VP) + VP) ^ VP) | X | VN
+        # Step 2: Computing HP and HN
+        HP = VN | ~(D0 | VP)
+        HN = D0 & VP
+        # Step 3: Computing the value D[m,j]
+        currDist += (HP & mask) != 0
+        currDist -= (HN & mask) != 0
+        # Step 4: Computing Vp and VN
+        HP = (HP << 1) | 1
+        HN = HN << 1
+        VP = HN | ~(D0 | HP)
+        VN = HP & D0
+
+        matrix_VP.append(VP)
+        matrix_VN.append(VN)
+
+    return (currDist, matrix_VP, matrix_VN)
+
+
 def editops(
     s1: Sequence[Hashable],
     s2: Sequence[Hashable],
@@ -425,7 +468,54 @@ def editops(
         s2 = processor(s2)
 
     s1, s2 = conv_sequences(s1, s2)
-    raise NotImplementedError
+    prefix_len, suffix_len = common_affix(s1, s2)
+    s1 = s1[prefix_len : len(s1) - suffix_len]
+    s2 = s2[prefix_len : len(s2) - suffix_len]
+    dist, VP, VN = _matrix(s1, s2)
+
+    editops = Editops([], 0, 0)
+    editops._src_len = len(s1) + prefix_len + suffix_len
+    editops._dest_len = len(s2) + prefix_len + suffix_len
+
+    if dist == 0:
+        return editops
+
+    editop_list: Any = [None] * dist
+    col = len(s1)
+    row = len(s2)
+    while row != 0 and col != 0:
+        # deletion
+        if VP[row - 1] & (1 << (col - 1)):
+            dist -= 1
+            col -= 1
+            editop_list[dist] = Editop("delete", col + prefix_len, row + prefix_len)
+        else:
+            row -= 1
+
+            # insertion
+            if row and (VN[row - 1] & (1 << (col - 1))):
+                dist -= 1
+                editop_list[dist] = Editop("insert", col + prefix_len, row + prefix_len)
+            else:
+                col -= 1
+
+                # replace (Matches are not recorded)
+                if s1[col] != s2[row]:
+                    dist -= 1
+                    editop_list[dist] = Editop("replace", col + prefix_len, row + prefix_len)
+
+    while col != 0:
+        dist -= 1
+        col -= 1
+        editop_list[dist] = Editop("delete", col + prefix_len, row + prefix_len)
+
+    while row != 0:
+        dist -= 1
+        row -= 1
+        editop_list[dist] = Editop("insert", col + prefix_len, row + prefix_len)
+
+    editops._editops = editop_list
+    return editops
 
 
 def opcodes(
