@@ -42,7 +42,7 @@ def _list_to_editops(
         blocks.append(Editop(edit_type, src_pos, dest_pos))
 
     # validate order of editops
-    for i in range(0, len(blocks) - 1):
+    for i in range(len(blocks) - 1):
         if blocks[i + 1].src_pos < blocks[i].src_pos or blocks[i + 1].dest_pos < blocks[i].dest_pos:
             msg = "List of edit operations out of order"
             raise ValueError(msg)
@@ -104,7 +104,7 @@ def _list_to_opcodes(
     if blocks[-1].src_end != src_len or blocks[-1].dest_end != dest_len:
         msg = "List of edit operations does not end at the string ends"
         raise ValueError(msg)
-    for i in range(0, len(blocks) - 1):
+    for i in range(len(blocks) - 1):
         if blocks[i + 1].src_start != blocks[i].src_end or blocks[i + 1].dest_start != blocks[i].dest_end:
             msg = "List of edit operations is not continuous"
             raise ValueError(msg)
@@ -342,7 +342,7 @@ class Editops:
 
         This is the equivalent of ``[x for x in editops]``
         """
-        return self._editops
+        return [tuple(op) for op in self._editops]
 
     def copy(self) -> Editops:
         """
@@ -472,15 +472,15 @@ class Editops:
 
         for op in self._editops:
             # matches between last and current editop
-            while src_pos < op.dest_pos:
+            while src_pos < op.src_pos:
                 res_str += source_string[src_pos]
                 src_pos += 1
 
             if op.tag == "replace":
-                res_str += destination_string[src_pos]
+                res_str += destination_string[op.dest_pos]
                 src_pos += 1
             elif op.tag == "insert":
-                res_str += destination_string[src_pos]
+                res_str += destination_string[op.dest_pos]
             elif op.tag == "delete":
                 src_pos += 1
 
@@ -618,7 +618,7 @@ class Opcode:
 
     def __repr__(self) -> str:
         return (
-            f"Opcode(tag={self.tag}, src_start={self.src_start}, src_end={self.src_end}, "
+            f"Opcode(tag={self.tag!r}, src_start={self.src_start}, src_end={self.src_end}, "
             f"dest_start={self.dest_start}, dest_end={self.dest_end})"
         )
 
@@ -711,7 +711,7 @@ class Opcodes:
 
         This is the equivalent of ``[x for x in opcodes]``
         """
-        return self._opcodes[::]
+        return [tuple(op) for op in self._opcodes]
 
     def copy(self) -> Opcodes:
         """
@@ -901,6 +901,6 @@ class ScoreAlignment:
 
 if TYPE_CHECKING:
     _AnyOpList = Union[
-        Sequence[Union[Editop, Tuple[str, int, int]]],
-        Sequence[Union[Opcode, Tuple[str, int, int, int, int]]],
+        Sequence[Union[Editop, tuple[str, int, int]]],
+        Sequence[Union[Opcode, tuple[str, int, int, int, int]]],
     ]
