@@ -6,7 +6,7 @@ from math import ceil
 from typing import Any, Callable, Hashable, Sequence
 
 from rapidfuzz._common_py import conv_sequences
-from rapidfuzz._utils import ScorerFlag, add_scorer_attrs, is_none
+from rapidfuzz._utils import ScorerFlag, add_scorer_attrs, is_none, setupPandas
 from rapidfuzz.distance import ScoreAlignment
 from rapidfuzz.distance.Indel_py import (
     _block_normalized_similarity as indel_block_normalized_similarity,
@@ -33,6 +33,36 @@ def _norm_distance(dist: int, lensum: int, score_cutoff: float) -> float:
     return score if score >= score_cutoff else 0
 
 
+def _split_sequence(seq: Sequence[Hashable]) -> list[Sequence[Hashable]]:
+    if isinstance(seq, (str, bytes)):
+        return seq.split()
+
+    splitted_seq = [[]]
+    for x in seq:
+        ch = x if isinstance(x, str) else chr(x)
+        if ch.isspace():
+            splitted_seq.append([])
+        else:
+            splitted_seq[-1].append(x)
+
+    return [tuple(x) for x in splitted_seq if x]
+
+
+def _join_splitted_sequence(seq_list: list[Sequence[Hashable]]):
+    if not seq_list:
+        return ""
+    if isinstance(next(iter(seq_list)), str):
+        return " ".join(seq_list)
+    if isinstance(next(iter(seq_list)), bytes):
+        return b" ".join(seq_list)
+
+    joined = []
+    for seq in seq_list:
+        joined += seq
+        joined += [ord(" ")]
+    return joined[:-1]
+
+
 def ratio(
     s1: Sequence[Hashable] | None,
     s2: Sequence[Hashable] | None,
@@ -75,6 +105,7 @@ def ratio(
     >>> fuzz.ratio("this is a test", "this is a test!")
     96.55171966552734
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -274,6 +305,7 @@ def partial_ratio_alignment(
     >>> fuzz.ratio(s1[res.src_start:res.src_end], s2[res.dest_start:res.dest_end])
     83.33333333333334
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return None
 
@@ -349,6 +381,7 @@ def token_sort_ratio(
     >>> fuzz.token_sort_ratio("fuzzy wuzzy was a bear", "wuzzy fuzzy was a bear")
     100.0
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -356,8 +389,9 @@ def token_sort_ratio(
         s1 = processor(s1)
         s2 = processor(s2)
 
-    sorted_s1 = " ".join(sorted(s1.split()))
-    sorted_s2 = " ".join(sorted(s2.split()))
+    s1, s2 = conv_sequences(s1, s2)
+    sorted_s1 = _join_splitted_sequence(sorted(_split_sequence(s1)))
+    sorted_s2 = _join_splitted_sequence(sorted(_split_sequence(s2)))
     return ratio(sorted_s1, sorted_s2, score_cutoff=score_cutoff)
 
 
@@ -402,6 +436,7 @@ def token_set_ratio(
     >>> fuzz.token_set_ratio("fuzzy was a bear", "fuzzy fuzzy was a bear")
     100.0
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -412,11 +447,13 @@ def token_set_ratio(
     if score_cutoff is None:
         score_cutoff = 0
 
-    tokens_a = set(s1.split())
-    tokens_b = set(s2.split())
+    s1, s2 = conv_sequences(s1, s2)
+
+    tokens_a = set(_split_sequence(s1))
+    tokens_b = set(_split_sequence(s2))
 
     # in FuzzyWuzzy this returns 0. For sake of compatibility return 0 here as well
-    # see https://github.com/maxbachmann/RapidFuzz/issues/110
+    # see https://github.com/rapidfuzz/RapidFuzz/issues/110
     if not tokens_a or not tokens_b:
         return 0
 
@@ -424,16 +461,17 @@ def token_set_ratio(
     diff_ab = tokens_a.difference(tokens_b)
     diff_ba = tokens_b.difference(tokens_a)
 
-    if not intersect and (not diff_ab or not diff_ba):
+    # one sentence is part of the other one
+    if intersect and (not diff_ab or not diff_ba):
         return 100
 
-    diff_ab_joined = " ".join(sorted(diff_ab))
-    diff_ba_joined = " ".join(sorted(diff_ba))
+    diff_ab_joined = _join_splitted_sequence(sorted(diff_ab))
+    diff_ba_joined = _join_splitted_sequence(sorted(diff_ba))
 
     ab_len = len(diff_ab_joined)
     ba_len = len(diff_ba_joined)
     # todo is length sum without joining faster?
-    sect_len = len(" ".join(intersect))
+    sect_len = len(_join_splitted_sequence(intersect))
 
     # string length sect+ab <-> sect and sect+ba <-> sect
     sect_ab_len = sect_len + (sect_len != 0) + ab_len
@@ -496,6 +534,7 @@ def token_ratio(
     -----
     .. image:: img/token_ratio.svg
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -543,6 +582,7 @@ def partial_token_sort_ratio(
     -----
     .. image:: img/partial_token_sort_ratio.svg
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -550,8 +590,9 @@ def partial_token_sort_ratio(
         s1 = processor(s1)
         s2 = processor(s2)
 
-    sorted_s1 = " ".join(sorted(s1.split()))
-    sorted_s2 = " ".join(sorted(s2.split()))
+    s1, s2 = conv_sequences(s1, s2)
+    sorted_s1 = _join_splitted_sequence(sorted(_split_sequence(s1)))
+    sorted_s2 = _join_splitted_sequence(sorted(_split_sequence(s2)))
     return partial_ratio(sorted_s1, sorted_s2, score_cutoff=score_cutoff)
 
 
@@ -589,6 +630,7 @@ def partial_token_set_ratio(
     -----
     .. image:: img/partial_token_set_ratio.svg
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -596,10 +638,12 @@ def partial_token_set_ratio(
         s1 = processor(s1)
         s2 = processor(s2)
 
-    tokens_a = set(s1.split())
-    tokens_b = set(s2.split())
+    s1, s2 = conv_sequences(s1, s2)
+
+    tokens_a = set(_split_sequence(s1))
+    tokens_b = set(_split_sequence(s2))
     # in FuzzyWuzzy this returns 0. For sake of compatibility return 0 here as well
-    # see https://github.com/maxbachmann/RapidFuzz/issues/110
+    # see https://github.com/rapidfuzz/RapidFuzz/issues/110
     if not tokens_a or not tokens_b:
         return 0
 
@@ -607,8 +651,8 @@ def partial_token_set_ratio(
     if tokens_a.intersection(tokens_b):
         return 100
 
-    diff_ab = " ".join(sorted(tokens_a.difference(tokens_b)))
-    diff_ba = " ".join(sorted(tokens_b.difference(tokens_a)))
+    diff_ab = _join_splitted_sequence(sorted(tokens_a.difference(tokens_b)))
+    diff_ba = _join_splitted_sequence(sorted(tokens_b.difference(tokens_a)))
     return partial_ratio(diff_ab, diff_ba, score_cutoff=score_cutoff)
 
 
@@ -646,6 +690,7 @@ def partial_token_ratio(
     -----
     .. image:: img/partial_token_ratio.svg
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -656,8 +701,10 @@ def partial_token_ratio(
     if score_cutoff is None:
         score_cutoff = 0
 
-    tokens_split_a = s1.split()
-    tokens_split_b = s2.split()
+    s1, s2 = conv_sequences(s1, s2)
+
+    tokens_split_a = _split_sequence(s1)
+    tokens_split_b = _split_sequence(s2)
     tokens_a = set(tokens_split_a)
     tokens_b = set(tokens_split_b)
 
@@ -669,8 +716,8 @@ def partial_token_ratio(
     diff_ba = tokens_b.difference(tokens_a)
 
     result = partial_ratio(
-        " ".join(sorted(tokens_split_a)),
-        " ".join(sorted(tokens_split_b)),
+        _join_splitted_sequence(sorted(tokens_split_a)),
+        _join_splitted_sequence(sorted(tokens_split_b)),
         score_cutoff=score_cutoff,
     )
 
@@ -682,8 +729,8 @@ def partial_token_ratio(
     return max(
         result,
         partial_ratio(
-            " ".join(sorted(diff_ab)),
-            " ".join(sorted(diff_ba)),
+            _join_splitted_sequence(sorted(diff_ab)),
+            _join_splitted_sequence(sorted(diff_ba)),
             score_cutoff=score_cutoff,
         ),
     )
@@ -722,6 +769,7 @@ def WRatio(
     -----
     .. image:: img/WRatio.svg
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -732,7 +780,7 @@ def WRatio(
         s2 = processor(s2)
 
     # in FuzzyWuzzy this returns 0. For sake of compatibility return 0 here as well
-    # see https://github.com/maxbachmann/RapidFuzz/issues/110
+    # see https://github.com/rapidfuzz/RapidFuzz/issues/110
     if not s1 or not s2:
         return 0
 
@@ -799,6 +847,7 @@ def QRatio(
     >>> fuzz.QRatio("this is a test", "this is a test!")
     96.55171966552734
     """
+    setupPandas()
     if is_none(s1) or is_none(s2):
         return 0
 
@@ -806,7 +855,7 @@ def QRatio(
         s1 = processor(s1)
         s2 = processor(s2)
     # in FuzzyWuzzy this returns 0. For sake of compatibility return 0 here as well
-    # see https://github.com/maxbachmann/RapidFuzz/issues/110
+    # see https://github.com/rapidfuzz/RapidFuzz/issues/110
     if not s1 or not s2:
         return 0
 
