@@ -15,9 +15,10 @@ from typing import (
     Mapping,
     Sequence,
     overload,
+    TYPE_CHECKING
 )
 
-from rapidfuzz._utils import ScorerFlag, is_none
+from rapidfuzz._utils import ScorerFlag, is_none, setupPandas
 from rapidfuzz.fuzz import WRatio, ratio
 
 __all__ = ["extract", "extract_iter", "extractOne", "cdist"]
@@ -41,8 +42,7 @@ def extract_iter(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> Iterable[tuple[Sequence[Hashable], int | float, int]]:
-    ...
+) -> Iterable[tuple[Sequence[Hashable], int | float, int]]: ...
 
 
 @overload
@@ -55,8 +55,7 @@ def extract_iter(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> Iterable[tuple[Sequence[Hashable], int | float, Any]]:
-    ...
+) -> Iterable[tuple[Sequence[Hashable], int | float, Any]]: ...
 
 
 def extract_iter(
@@ -103,7 +102,7 @@ def extract_iter(
 
     Yields
     -------
-    Tuple[Sequence[Hashable], Any, Any]
+    tuple[Sequence[Hashable], Any, Any]
         Yields similarity between the query and each choice in form of a Tuple with 3 elements.
         The values stored in the tuple depend on the types of the input arguments.
 
@@ -131,6 +130,8 @@ def extract_iter(
     worst_score, optimal_score = _get_scorer_flags_py(scorer, scorer_kwargs)
     lowest_score_worst = optimal_score > worst_score
 
+    setupPandas()
+
     if is_none(query):
         return
 
@@ -175,8 +176,7 @@ def extractOne(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> tuple[Sequence[Hashable], int | float, int] | None:
-    ...
+) -> tuple[Sequence[Hashable], int | float, int] | None: ...
 
 
 @overload
@@ -189,8 +189,7 @@ def extractOne(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> tuple[Sequence[Hashable], int | float, Any] | None:
-    ...
+) -> tuple[Sequence[Hashable], int | float, Any] | None: ...
 
 
 def extractOne(
@@ -238,7 +237,7 @@ def extractOne(
 
     Returns
     -------
-    Tuple[Sequence[Hashable], Any, Any]
+    tuple[Sequence[Hashable], Any, Any]
         Returns the best match in form of a Tuple with 3 elements. The values stored in the
         tuple depend on the types of the input arguments.
 
@@ -324,6 +323,8 @@ def extractOne(
     worst_score, optimal_score = _get_scorer_flags_py(scorer, scorer_kwargs)
     lowest_score_worst = optimal_score > worst_score
 
+    setupPandas()
+
     if is_none(query):
         return None
 
@@ -378,8 +379,7 @@ def extract(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> list[tuple[Sequence[Hashable], int | float, int]]:
-    ...
+) -> list[tuple[Sequence[Hashable], int | float, int]]: ...
 
 
 @overload
@@ -393,8 +393,7 @@ def extract(
     score_cutoff: int | float | None = None,
     score_hint: int | float | None = None,
     scorer_kwargs: dict[str, Any] | None = None,
-) -> list[tuple[Sequence[Hashable], int | float, Any]]:
-    ...
+) -> list[tuple[Sequence[Hashable], int | float, Any]]: ...
 
 
 def extract(
@@ -446,7 +445,7 @@ def extract(
 
     Returns
     -------
-    List[Tuple[Sequence[Hashable], Any, Any]]
+    list[tuple[Sequence[Hashable], Any, Any]]
         The return type is always a List of Tuples with 3 elements. However the values stored in the
         tuple depend on the types of the input arguments.
 
@@ -508,8 +507,9 @@ def extract(
     return heapq.nsmallest(limit, result_iter, key=lambda i: i[1])
 
 
-with suppress(BaseException):
-    import numpy as np
+if TYPE_CHECKING:
+    with suppress(BaseException):
+        import numpy as np
 
 
 def _dtype_to_type_num(
@@ -527,6 +527,8 @@ def _dtype_to_type_num(
         flags = params["get_scorer_flags"](**scorer_kwargs)
         if flags["flags"] & ScorerFlag.RESULT_I64:
             return np.int32
+        if flags["flags"] & ScorerFlag.RESULT_SIZE_T:
+            return np.uint32
         return np.float32
 
     return np.float32
@@ -618,6 +620,8 @@ def cdist(
     dtype = _dtype_to_type_num(dtype, scorer, scorer_kwargs)
     results = np.zeros((len(queries), len(choices)), dtype=dtype)
 
+    setupPandas()
+
     if processor is None:
         proc_choices = list(choices)
     else:
