@@ -4,6 +4,7 @@ import pytest
 
 from rapidfuzz import fuzz, process_cpp, process_py
 from rapidfuzz.distance import Levenshtein, Levenshtein_py
+from rapidfuzz.utils import default_process
 
 
 def wrapped(func):
@@ -192,6 +193,28 @@ def test_with_processor():
     best = process.extractOne(query, events, processor=lambda event: event[0])
     assert best[0] == events[0]
 
+    best = process.extract(query, events, processor=lambda event: event[0])[0]
+    assert best[0] == events[0]
+
+    eventsDict = {i: elem for i, elem in enumerate(events)}
+    best = process.extractOne(query, eventsDict, processor=lambda event: event[0])
+    assert best[0] == events[0]
+
+    best = process.extract(query, eventsDict, processor=lambda event: event[0])[0]
+    assert best[0] == events[0]
+
+    best = process.extractOne("new york mets", ["new YORK mets"])
+    assert 72 < best[1] < 73
+
+    best = process.extract("new york mets", ["new YORK mets"])[0]
+    assert 72 < best[1] < 73
+
+    best = process.extractOne("new york mets", ["new YORK mets"], processor=default_process)
+    assert best[1] == 100
+
+    best = process.extract("new york mets", ["new YORK mets"], processor=default_process)[0]
+    assert best[1] == 100
+
 
 def test_with_scorer():
     choices = [
@@ -278,15 +301,65 @@ def test_none_elements():
     """
     when a None element is used, it is skipped and the index is still correct
     """
+    # no processor
     best = process.extractOne("test", [None, "tes"])
     assert best[2] == 1
     best = process.extractOne(None, [None, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [None, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(None, [None, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [None, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(None, [None, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [None, "tes"])
     assert best[0][2] == 1
     best = process.extract(None, [None, "tes"])
     assert best == []
+    best = process.extract("test", {0: None, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [None, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(None, [None, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: None, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [None, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(None, [None, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_numpy_nan_elements():
@@ -294,15 +367,65 @@ def test_numpy_nan_elements():
     when a np.nan element is used, it is skipped and the index is still correct
     """
     np = pytest.importorskip("numpy")
+    # no processor
     best = process.extractOne("test", [np.nan, "tes"])
     assert best[2] == 1
     best = process.extractOne(np.nan, [np.nan, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [np.nan, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, [np.nan, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [np.nan, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, [np.nan, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [np.nan, "tes"])
     assert best[0][2] == 1
     best = process.extract(np.nan, [np.nan, "tes"])
     assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [np.nan, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, [np.nan, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [np.nan, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, [np.nan, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_pandas_nan_elements():
@@ -310,15 +433,65 @@ def test_pandas_nan_elements():
     when a pd.NA element is used, it is skipped and the index is still correct
     """
     pd = pytest.importorskip("pandas")
+    # no processor
     best = process.extractOne("test", [pd.NA, "tes"])
     assert best[2] == 1
     best = process.extractOne(pd.NA, [pd.NA, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [pd.NA, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, [pd.NA, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [pd.NA, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, [pd.NA, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [pd.NA, "tes"])
     assert best[0][2] == 1
     best = process.extract(pd.NA, [pd.NA, "tes"])
     assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [pd.NA, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, [pd.NA, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [pd.NA, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, [pd.NA, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_result_order():
@@ -510,7 +683,7 @@ def test_generators():
 
 def test_cdist_pure_python_dtype():
     np = pytest.importorskip("numpy")
-    assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.distance).dtype == np.int32
-    assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.similarity).dtype == np.int32
+    assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.distance).dtype == np.uint32
+    assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.similarity).dtype == np.uint32
     assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.normalized_distance).dtype == np.float32
     assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.normalized_similarity).dtype == np.float32
